@@ -174,7 +174,7 @@ module Sequel
     #   # => false
     def empty?
       cached_dataset(:_empty_ds) do
-        single_value_ds.unordered.select(EMPTY_SELECT)
+        (@opts[:sql] ? from_self : self).single_value_ds.unordered.select(EMPTY_SELECT)
       end.single_value!.nil?
     end
 
@@ -217,7 +217,7 @@ module Sequel
       case args.length
       when 0
         unless block
-          return single_record
+          return(@opts[:sql] ? single_record! : single_record)
         end
       when 1
         arg = args[0]
@@ -282,6 +282,12 @@ module Sequel
     #
     #   DB[:table].get{[sum(id).as(sum), name]} # SELECT sum(id) AS sum, name FROM table LIMIT 1
     #   # => [6, 'foo']
+    #
+    # If called on a dataset with raw SQL, returns the
+    # first value in the dataset without changing the selection or setting a limit:
+    #
+    #   DB["SELECT id FROM table"].get # SELECT id FROM table
+    #   # =>  3
     def get(column=(no_arg=true; nil), &block)
       ds = naked
       if block
@@ -289,6 +295,8 @@ module Sequel
         ds = ds.select(&block)
         column = ds.opts[:select]
         column = nil if column.is_a?(Array) && column.length < 2
+      elsif no_arg && opts[:sql]
+        return ds.single_value!
       else
         case column
         when Array
@@ -356,9 +364,11 @@ module Sequel
     #            This does not have an effect if +values+ is a Dataset.
     # :server :: Set the server/shard to use for the transaction and insert
     #            queries.
+    # :skip_transaction :: Do not use a transaction even when using multiple
+    #                      INSERT queries.
     # :slice :: Same as :commit_every, :commit_every takes precedence.
     def import(columns, values, opts=OPTS)
-      return @db.transaction{insert(columns, values)} if values.is_a?(Dataset)
+      return insert(columns, values) if values.is_a?(Dataset)
 
       return if values.empty?
       raise(Error, 'Using Sequel::Dataset#import with an empty column array is not allowed') if columns.empty?
@@ -588,6 +598,8 @@ module Sequel
     #                   if your ORDER BY expressions are not simple columns, if they contain
     #                   qualified identifiers that would be ambiguous unqualified, if they contain
     #                   any identifiers that are aliased in SELECT, and potentially other cases.
+    # :skip_transaction :: Do not use a transaction. This can be useful if you want to prevent
+    #                      a lock on the database table, at the expense of consistency.
     #
     # Examples:
     #
@@ -1111,11 +1123,9 @@ module Sequel
     # are provided. When only a single value or statement is provided, then yield
     # without using a transaction.
     def _import_transaction(values, trans_opts, &block)
-      if values.length > 1
-        @db.transaction(trans_opts, &block)
-      else
-        yield
-      end
+      # OK to mutate trans_opts as it is generated by _import
+      trans_opts[:skip_transaction] = true if values.length <= 1
+      @db.transaction(trans_opts, &block)
     end
   
     # Internals of +select_hash+ and +select_hash_groups+
@@ -1246,8 +1256,6 @@ module Sequel
     # receiver's current order. This yields the row and the array of order expressions
     # to the block, which should return an array of values to use.
     def ignore_values_preceding(row)
-      @opts[:order].map{|v| v.is_a?(SQL::OrderedExpression) ? v.expression : v}
-
       order_exprs = @opts[:order].map do |v|
         if v.is_a?(SQL::OrderedExpression)
           descending = v.descending
