@@ -102,7 +102,7 @@ class String
   # Yield the Inflections module if a block is given, and return
   # the Inflections module.
   def self.inflections
-    yield Inflections if block_given?
+    yield Inflections if defined?(yield)
     Inflections
   end
   
@@ -159,7 +159,7 @@ class String
   # Example
   #   "puni_puni".dasherize #=> "puni-puni"
   def dasherize
-    gsub('_', '-')
+    tr('_', '-')
   end
 
   # Removes the module part from the expression in the string
@@ -189,7 +189,7 @@ class String
   #   "employee_salary" #=> "Employee salary"
   #   "author_id" #=> "Author"
   def humanize
-    gsub(/_id$/, "").gsub('_', " ").capitalize
+    gsub(/_id$/, "").tr('_', " ").capitalize
   end
 
   # Returns the plural form of the word in the string.
