@@ -145,7 +145,7 @@ describe "Sequel Mock Adapter" do
     rs.must_equal [{:a=>1}] * 2
   end
 
-  it "should be able to set an exception to raise by setting the :fetch option to an exception class " do
+  it "should be able to set an exception to raise by setting the :fetch option to an exception class" do
     db = Sequel.mock(:fetch=>ArgumentError)
     proc{db[:t].all}.must_raise(Sequel::DatabaseError)
     begin
@@ -212,7 +212,7 @@ describe "Sequel Mock Adapter" do
     db[:b].delete.must_equal 1
   end
 
-  it "should be able to set an exception to raise by setting the :numrows option to an exception class " do
+  it "should be able to set an exception to raise by setting the :numrows option to an exception class" do
     db = Sequel.mock(:numrows=>ArgumentError)
     proc{db[:t].update(:a=>1)}.must_raise(Sequel::DatabaseError)
     begin
@@ -280,7 +280,7 @@ describe "Sequel Mock Adapter" do
     db[:b].insert(:a=>1).must_equal 1
   end
 
-  it "should be able to set an exception to raise by setting the :autoid option to an exception class " do
+  it "should be able to set an exception to raise by setting the :autoid option to an exception class" do
     db = Sequel.mock(:autoid=>ArgumentError)
     proc{db[:t].insert(:a=>1)}.must_raise(Sequel::DatabaseError)
     begin
@@ -424,6 +424,12 @@ describe "Sequel Mock Adapter" do
     db.sqls.must_equal ['BEGIN', 'BEGIN -- "test"', 'SELECT * FROM a', 'SELECT * FROM t -- "test"', 'COMMIT -- "test"', 'COMMIT']
   end
 
+  it "should correctly handle with_advisory_lock when sharding" do
+    db = Sequel.mock(:host=>'postgres', :servers=>{:test=>{}, 'test'=>{}}, :fetch=>{:v=>true})
+    db.with_advisory_lock(1234, server: :test){}
+    db.sqls.must_equal ['SELECT pg_try_advisory_lock(1234) LIMIT 1 -- test', 'SELECT pg_advisory_unlock(1234) LIMIT 1 -- test']
+  end
+
   it "should yield a mock connection object from synchronize" do
     c = Sequel.mock.synchronize{|conn| conn}
     c.must_be_kind_of(Sequel::Mock::Connection)
@@ -523,6 +529,35 @@ describe "PostgreSQL support" do
     @db.sqls.must_equal ['CREATE UNLOGGED TABLE "unlogged_dolls" ("name" text)']
   end
 
+  it "should respect :unlogged_tables_default Database option when creating a table" do
+    @db.opts[:unlogged_tables_default] = true
+    @db.create_table(:unlogged_dolls, :unlogged => true){text :name}
+    @db.sqls.must_equal ['CREATE UNLOGGED TABLE "unlogged_dolls" ("name" text)']
+    @db.create_table(:unlogged_dolls){text :name}
+    @db.sqls.must_equal ['CREATE UNLOGGED TABLE "unlogged_dolls" ("name" text)']
+    @db.create_table(:unlogged_dolls, :unlogged => false){text :name}
+    @db.sqls.must_equal ['CREATE TABLE "unlogged_dolls" ("name" text)']
+
+    @db.opts[:unlogged_tables_default] = 't'
+    @db.create_table(:unlogged_dolls){text :name}
+    @db.sqls.must_equal ['CREATE UNLOGGED TABLE "unlogged_dolls" ("name" text)']
+
+    @db.opts[:unlogged_tables_default] = 'f'
+    @db.create_table(:unlogged_dolls){text :name}
+    @db.sqls.must_equal ['CREATE TABLE "unlogged_dolls" ("name" text)']
+  end
+
+  it "should respect :deferrable when creating unique constraints" do
+    @db.create_table(:t){unique :x, deferrable: true}
+    @db.sqls.must_equal ['CREATE TABLE "t" (UNIQUE ("x") DEFERRABLE INITIALLY DEFERRED)']
+
+    @db.alter_table(:t){add_unique_constraint :x, deferrable: true}
+    @db.sqls.must_equal ['ALTER TABLE "t" ADD UNIQUE ("x") DEFERRABLE INITIALLY DEFERRED']
+
+    @db.create_table(:t){Integer :x, unique: {deferrable: true}}
+    @db.sqls.must_equal ['CREATE TABLE "t" ("x" integer UNIQUE DEFERRABLE INITIALLY DEFERRED)']
+  end
+
   it "should support spatial indexes" do
     @db.alter_table(:posts){add_spatial_index [:geom]}
     @db.sqls.must_equal ['CREATE INDEX "posts_geom_index" ON "posts" USING gist ("geom")']
@@ -712,6 +747,32 @@ describe "PostgreSQL support" do
     @db.sqls.must_equal ["ALTER TABLE \"t\" DROP COLUMN IF EXISTS \"x\" CASCADE"]
   end
 
+  it "should support :include option for unique and primary key constraints" do
+    @db.create_table(:t){Integer :x, :primary_key => {:include => :y}; Integer :y}
+    @db.sqls.must_equal ['CREATE TABLE "t" ("x" integer PRIMARY KEY INCLUDE ("y"), "y" integer)']
+
+    @db.create_table(:t){Integer :x, :unique => {:include => :y}; Integer :y}
+    @db.sqls.must_equal ['CREATE TABLE "t" ("x" integer UNIQUE INCLUDE ("y"), "y" integer)']
+
+    @db.create_table(:t){Integer :x; primary_key [:x], :include => :y; Integer :y}
+    @db.sqls.must_equal ['CREATE TABLE "t" ("x" integer, "y" integer, PRIMARY KEY ("x") INCLUDE ("y"))']
+
+    @db.create_table(:t){Integer :x; unique [:x], {:include => :y}; Integer :y}
+    @db.sqls.must_equal ['CREATE TABLE "t" ("x" integer, "y" integer, UNIQUE ("x") INCLUDE ("y"))']
+
+    @db.alter_table(:t){add_primary_key [:x], :include => :y}
+    @db.sqls.must_equal ['ALTER TABLE "t" ADD PRIMARY KEY ("x") INCLUDE ("y")']
+
+    @db.alter_table(:t){add_unique_constraint [:x], :include => :y}
+    @db.sqls.must_equal ['ALTER TABLE "t" ADD UNIQUE ("x") INCLUDE ("y")']
+
+    @db.create_table(:t){Integer :x; exclude [[Sequel.desc(:x, :nulls=>:last), '=']], :using=>:btree, :where=>proc{x > 0}, :include => [:y]}
+    @db.sqls.must_equal ['CREATE TABLE "t" ("x" integer, EXCLUDE USING btree ("x" DESC NULLS LAST WITH =) INCLUDE ("y") WHERE ("x" > 0))']
+
+    @db.alter_table(:t){add_exclusion_constraint [[Sequel.desc(:x, :nulls=>:last), '=']], :using=>:btree, :where=>proc{x > 0}, :include => [:y], name: "c"}
+    @db.sqls.must_equal ['ALTER TABLE "t" ADD CONSTRAINT "c" EXCLUDE USING btree ("x" DESC NULLS LAST WITH =) INCLUDE ("y") WHERE ("x" > 0)']
+  end
+
   it "should support transaction isolation levels" do
     @db.supports_transaction_isolation_levels?.must_equal true
   end
@@ -825,6 +886,16 @@ describe "PostgreSQL support" do
     ]
   end
 
+  it "should default generated columns to STORED" do
+    @db.create_table(:t){Integer :c, :generated_always_as=>Sequel[:t] + 1}
+    @db.sqls.must_equal ['CREATE TABLE "t" ("c" integer GENERATED ALWAYS AS (("t" + 1)) STORED)']
+  end
+
+  it "should create VIRTUAL generated columns if given :virtual option" do
+    @db.create_table(:t){Integer :c, :generated_always_as=>Sequel[:t] + 1, :virtual=>true}
+    @db.sqls.must_equal ['CREATE TABLE "t" ("c" integer GENERATED ALWAYS AS (("t" + 1)) VIRTUAL)']
+  end
+
   it "should not support converting serial columns to identity on PostgreSQL <10.2" do
     def @db.server_version; 100000; end
     proc{@db.convert_serial_to_identity(:table)}.must_raise Sequel::Error
@@ -869,6 +940,66 @@ describe "PostgreSQL support" do
   it "should recognize 40P01 SQL state as a serialization failure" do
     @db.send(:database_specific_error_class_from_sqlstate, '40P01').must_equal Sequel::SerializationFailure
   end
+
+  it "should use correct SQL for defer_constraints and immediate_constraints" do
+    @db.defer_constraints
+    @db.sqls.must_equal ['SET CONSTRAINTS ALL DEFERRED']
+    @db.immediate_constraints
+    @db.sqls.must_equal ['SET CONSTRAINTS ALL IMMEDIATE']
+
+    @db.defer_constraints(:constraints=>:a)
+    @db.sqls.must_equal ['SET CONSTRAINTS "a" DEFERRED']
+    @db.immediate_constraints(:constraints=>[:a, :b])
+    @db.sqls.must_equal ['SET CONSTRAINTS "a", "b" IMMEDIATE']
+  end
+
+  it "should correctly handle defer_constraints and immediate_constraints :server option" do
+    db = Sequel.connect("mock://postgres", :servers=>{:test=>{}})
+    db.defer_constraints(:server=>:test)
+    db.sqls.must_equal ['SET CONSTRAINTS ALL DEFERRED -- test']
+    db.immediate_constraints(:server=>:test)
+    db.sqls.must_equal ['SET CONSTRAINTS ALL IMMEDIATE -- test']
+  end
+
+  it "should raise if an invalid number of elements is provided when using derived column lists with an array" do
+    @db.from{c.function.as(:d, [[:a, :b], [:e, :f]])}.sql.must_equal 'SELECT * FROM c() AS "d"("a" b, "e" f)'
+    proc{@db.from{c.function.as(:d, [[:a], [:e, :f]])}.sql}.must_raise Sequel::Error
+    proc{@db.from{c.function.as(:d, [[:a, :b, :c], [:e, :f]])}.sql}.must_raise Sequel::Error
+  end
+
+  it "should support :using_index option when adding a UNIQUE constraint" do
+    @db.create_table(:posts) do
+      Integer :i
+      index :i, :name=>:posts_i_uidx, :unique=>true
+    end
+
+    @db.sqls
+    @db.alter_table(:posts) do
+      add_unique_constraint :i, :using_index=>:posts_i_uidx
+    end
+    @db.sqls.must_equal ['ALTER TABLE "posts" ADD UNIQUE USING INDEX "posts_i_uidx"']
+  end
+
+  it "should support :using_index option when adding a PRIMARY KEY constraint" do
+    @db.create_table(:posts) do
+      Integer :i
+      index :i, :name=>:posts_i_uidx, :unique=>true
+    end
+
+    @db.sqls
+    @db.alter_table(:posts) do
+      add_primary_key [:i], :using_index=>:posts_i_uidx, :name=>"posts_pkey"
+    end
+    @db.sqls.must_equal ['ALTER TABLE "posts" ADD CONSTRAINT "posts_pkey" PRIMARY KEY USING INDEX "posts_i_uidx"']
+  end
+
+  it "should handle :key as a symbol when setting up a foreign key constraint" do
+    @db.create_table(:a){foreign_key :x, :t, :key => :y}
+    @db.sqls.must_equal ['CREATE TABLE "a" ("x" integer REFERENCES "t"("y"))']
+
+    @db.create_table(:a){foreign_key [:x], :t, :key => :y}
+    @db.sqls.must_equal ['CREATE TABLE "a" (FOREIGN KEY ("x") REFERENCES "t"("y"))']
+  end
 end
 
 describe "MySQL support" do
