@@ -1,5 +1,3 @@
-require_relative "../sequel_warning"
-
 if ENV['COVERAGE']
   require_relative "../sequel_coverage"
   SimpleCov.sequel_coverage(:filter=>%r{lib/sequel/(\w+\.rb|(dataset|database|model|connection_pool)/\w+\.rb|adapters/mock\.rb)\z})
@@ -8,10 +6,10 @@ end
 $:.unshift(File.join(File.dirname(File.expand_path(__FILE__)), "../../lib/"))
 require_relative "../../lib/sequel/core"
 
+ENV['MT_NO_PLUGINS'] = '1' # Work around stupid autoloading of plugins
 gem 'minitest'
-require 'minitest/autorun'
+require 'minitest/global_expectations/autorun'
 require 'minitest/hooks/default'
-require 'minitest/shared_description'
 
 require_relative '../deprecation_helper'
 
