@@ -1,5 +1,3 @@
-require_relative "sequel_warning"
-
 if ENV['COVERAGE']
   require_relative "sequel_coverage"
   SimpleCov.sequel_coverage(:filter=>%r{lib/sequel/extensions/core_extensions\.rb\z})
@@ -592,10 +590,10 @@ describe "Symbol" do
   end
 end
 
-describe "Postgres extensions integration" do
+describe "Postgres/SQLite extensions integration" do
   before do
     @db = Sequel.mock
-    Sequel.extension(:pg_array, :pg_array_ops, :pg_hstore, :pg_hstore_ops, :pg_json, :pg_json_ops, :pg_range, :pg_range_ops, :pg_row, :pg_row_ops, :pg_inet_ops)
+    Sequel.extension(:pg_array, :pg_array_ops, :pg_hstore, :pg_hstore_ops, :pg_json, :pg_json_ops, :pg_range, :pg_range_ops, :pg_row, :pg_row_ops, :pg_inet_ops, :sqlite_json_ops)
   end
 
   it "Symbol#pg_array should return an ArrayOp" do
@@ -661,6 +659,19 @@ describe "Postgres extensions integration" do
     @db.literal((1..2).pg_range).must_equal "'[1,2]'"
     @db.literal((1..2).pg_range(:int4range)).must_equal "int4range(1,2,'[]')"
   end
+
+  it "Symbol#sqlite_json_opt should return an SQLite::JSONOp" do
+    @db.literal(:a.sqlite_json_op[1]).must_equal "(a ->> 1)"
+    @db.literal(:a.sqlite_json_op.minify).must_equal "json(a)"
+  end
+end
+
+describe "is_distinct_from extension integration" do
+  it "Symbol#is_distinct_from should return an IsDistinctFrom" do
+    db = Sequel.connect("mock://postgres")
+    db.extension :is_distinct_from
+    db.literal(:a.is_distinct_from(:b)).must_equal '("a" IS DISTINCT FROM "b")'
+  end
 end
 
 describe "symbol_aref extensions" do
