@@ -5,8 +5,8 @@ describe "class_table_inheritance plugin" do
     @db = Sequel.mock(:numrows=>1, :autoid=>proc{|sql| 1})
     def @db.supports_schema_parsing?() true end
     def @db.schema(table, opts={})
-      {:employees=>[[:id, {:primary_key=>true, :type=>:integer}], [:name, {:type=>:string}], [:kind, {:type=>:string}]],
-       :managers=>[[:id, {:type=>:integer}], [:num_staff, {:type=>:integer}] ],
+      {:employees=>[[:id, {:primary_key=>true, :type=>:integer}], [:name, {:type=>:string, :allow_null=>false}], [:kind, {:type=>:string}]],
+       :managers=>[[:id, {:type=>:integer}], [:num_staff, {:type=>:integer, :allow_null=>false}] ],
        :executives=>[[:id, {:type=>:integer}], [:num_managers, {:type=>:integer}]],
        :staff=>[[:id, {:type=>:integer}], [:manager_id, {:type=>:integer}]],
        }[table.is_a?(Sequel::Dataset) ? table.first_source_table : table]
@@ -23,11 +23,14 @@ describe "class_table_inheritance plugin" do
         }[opts[:from] + (opts[:join] || []).map{|x| x.table}]
       end
     end
-    class ::Employee < Sequel::Model(@db)
-      def _save_refresh; @values[:id] = 1 end
+    base = Sequel::Model(@db)
+    base.plugin :auto_validations if @use_auto_validations
+    class ::Employee < base
       def self.columns
         dataset.columns || dataset.opts[:from].first.expression.columns
       end
+      private
+      def _save_refresh; @values[:id] = 1 end
       plugin :class_table_inheritance, :key=>:kind, :table_map=>{:Staff=>:staff}
     end 
     class ::Manager < Employee
@@ -94,6 +97,15 @@ describe "class_table_inheritance plugin" do
     Staff.dataset.sql.must_equal 'SELECT * FROM (SELECT employees.id, employees.name, employees.kind, staff.manager_id FROM employees INNER JOIN staff ON (staff.id = employees.id)) AS employees'
     Intern.dataset.sql.must_equal 'SELECT * FROM employees WHERE (employees.kind IN (\'Intern\'))'
   end
+
+  it "should use a empty empty string for class name for anonymous subclasses" do
+    Class.new(Employee).dataset.sql.must_equal "SELECT * FROM employees WHERE (employees.kind IN (''))"
+    Class.new(Manager).dataset.sql.must_equal "SELECT * FROM (SELECT employees.id, employees.name, employees.kind, managers.num_staff FROM employees INNER JOIN managers ON (managers.id = employees.id) WHERE (employees.kind IN (''))) AS employees"
+    Class.new(Executive).dataset.sql.must_equal "SELECT * FROM (SELECT employees.id, employees.name, employees.kind, managers.num_staff, executives.num_managers FROM employees INNER JOIN managers ON (managers.id = employees.id) INNER JOIN executives ON (executives.id = managers.id) WHERE (employees.kind IN (''))) AS employees"
+    Class.new(Ceo).dataset.sql.must_equal "SELECT * FROM (SELECT employees.id, employees.name, employees.kind, managers.num_staff, executives.num_managers FROM employees INNER JOIN managers ON (managers.id = employees.id) INNER JOIN executives ON (executives.id = managers.id) WHERE (employees.kind IN (''))) AS employees"
+    Class.new(Staff).dataset.sql.must_equal "SELECT * FROM (SELECT employees.id, employees.name, employees.kind, staff.manager_id FROM employees INNER JOIN staff ON (staff.id = employees.id) WHERE (employees.kind IN (''))) AS employees"
+    Class.new(Intern).dataset.sql.must_equal "SELECT * FROM (SELECT * FROM employees WHERE (employees.kind IN (''))) AS employees"
+  end
   
   it "should return rows with the correct class based on the polymorphic_key value" do
     @ds.with_fetch([{:kind=>'Employee'}, {:kind=>'Manager'}, {:kind=>'Executive'}, {:kind=>'Ceo'}, {:kind=>'Staff'}, {:kind=>'Intern'}]).all.collect{|x| x.class}.must_equal [Employee, Manager, Executive, Ceo, Staff, Intern]
@@ -114,6 +126,26 @@ describe "class_table_inheritance plugin" do
       "SELECT * FROM (SELECT employees.id, employees.name, employees.kind, managers.num_staff, executives.num_managers FROM employees INNER JOIN managers ON (managers.id = employees.id) INNER JOIN executives ON (executives.id = managers.id) WHERE (employees.kind IN ('Ceo'))) AS employees WHERE (id = 1) LIMIT 1"]
   end
   
+  describe "with auto_validations plugin" do
+    before(:all) do
+      @use_auto_validations = true
+    end
+
+    it "should work" do
+      e = Employee.new
+      e.valid?.must_equal false
+      e.errors.must_equal(:name=>["is not present"])
+
+      e = Manager.new
+      e.valid?.must_equal false
+      e.errors.must_equal(:name=>["is not present"], :num_staff=>["is not present"])
+
+      e = Executive.new
+      e.valid?.must_equal false
+      e.errors.must_equal(:name=>["is not present"], :num_staff=>["is not present"])
+    end
+  end
+    
   it "should return rows with the current class if sti_key is nil" do
     Employee.plugin :class_table_inheritance
     Employee.dataset.with_fetch([{:kind=>'Employee'}, {:kind=>'Manager'}, {:kind=>'Executive'}, {:kind=>'Ceo'}, {:kind=>'Staff'}, {:kind=>'Intern'}]).all.map{|x| x.class}.must_equal [Employee, Employee, Employee, Employee, Employee, Employee]
@@ -193,6 +225,21 @@ describe "class_table_inheritance plugin" do
     @db.sqls.must_equal ["INSERT INTO employees (kind) VALUES ('Employee')"]
   end
 
+  it "should handle type field matching current class when saving" do
+    Employee.create(:kind=>'Employee')
+    @db.sqls.must_equal ["INSERT INTO employees (kind) VALUES ('Employee')"]
+  end
+
+  it "should convert type field matching subclass using different table when saving" do
+    Employee.create(:kind=>'Manager')
+    @db.sqls.must_equal ["INSERT INTO employees (kind) VALUES ('Employee')"]
+  end
+
+  it "should keep type field matching subclass using same table when saving" do
+    Employee.create(:kind=>'Intern')
+    @db.sqls.must_equal ["INSERT INTO employees (kind) VALUES ('Intern')"]
+  end
+
   it "should allow specifying a map of names to tables to override implicit mapping" do
     Manager.dataset.sql.must_equal 'SELECT * FROM (SELECT employees.id, employees.name, employees.kind, managers.num_staff FROM employees INNER JOIN managers ON (managers.id = employees.id)) AS employees'
     Staff.dataset.sql.must_equal 'SELECT * FROM (SELECT employees.id, employees.name, employees.kind, staff.manager_id FROM employees INNER JOIN staff ON (staff.id = employees.id)) AS employees'
@@ -232,11 +279,11 @@ describe "class_table_inheritance plugin" do
   end
   
   it "should include schema for columns for tables for ancestor classes" do
-    Employee.db_schema.must_equal(:id=>{:primary_key=>true, :type=>:integer}, :name=>{:type=>:string}, :kind=>{:type=>:string})
-    Manager.db_schema.must_equal(:id=>{:primary_key=>true, :type=>:integer}, :name=>{:type=>:string}, :kind=>{:type=>:string}, :num_staff=>{:type=>:integer})
-    Executive.db_schema.must_equal(:id=>{:primary_key=>true, :type=>:integer}, :name=>{:type=>:string}, :kind=>{:type=>:string}, :num_staff=>{:type=>:integer}, :num_managers=>{:type=>:integer})
-    Staff.db_schema.must_equal(:id=>{:primary_key=>true, :type=>:integer}, :name=>{:type=>:string}, :kind=>{:type=>:string}, :manager_id=>{:type=>:integer})
-    Intern.db_schema.must_equal(:id=>{:primary_key=>true, :type=>:integer}, :name=>{:type=>:string}, :kind=>{:type=>:string})
+    Employee.db_schema.must_equal(:id=>{:primary_key=>true, :type=>:integer}, :name=>{:type=>:string, :allow_null=>false}, :kind=>{:type=>:string})
+    Manager.db_schema.must_equal(:id=>{:primary_key=>true, :type=>:integer}, :name=>{:type=>:string, :allow_null=>false}, :kind=>{:type=>:string}, :num_staff=>{:type=>:integer, :allow_null=>false})
+    Executive.db_schema.must_equal(:id=>{:primary_key=>true, :type=>:integer}, :name=>{:type=>:string, :allow_null=>false}, :kind=>{:type=>:string}, :num_staff=>{:type=>:integer, :allow_null=>false}, :num_managers=>{:type=>:integer})
+    Staff.db_schema.must_equal(:id=>{:primary_key=>true, :type=>:integer}, :name=>{:type=>:string, :allow_null=>false}, :kind=>{:type=>:string}, :manager_id=>{:type=>:integer})
+    Intern.db_schema.must_equal(:id=>{:primary_key=>true, :type=>:integer}, :name=>{:type=>:string, :allow_null=>false}, :kind=>{:type=>:string})
   end
 
   it "should use the correct primary key (which should have the same name in all subclasses)" do
@@ -329,6 +376,12 @@ describe "class_table_inheritance plugin" do
     @db.sqls.must_equal ["UPDATE employees SET name = 'E' WHERE (id = 2)", "UPDATE managers SET num_staff = 2 WHERE (id = 2)", "UPDATE executives SET num_managers = 3 WHERE (id = 2)"]
   end
 
+  it "should update only tables with changes when updating" do
+    obj = Ceo.load(:id=>2, :num_managers=>3, :num_staff=>2, :name=>'E')
+    obj.update(:num_staff=>3)
+    @db.sqls.must_equal ["UPDATE managers SET num_staff = 3 WHERE (id = 2)"]
+  end
+
   it "should raise error if one of the updates does not update a single row" do
     @db.numrows = [1, 0]
     proc{Ceo.load(:id=>2).update(:num_managers=>3, :num_staff=>2, :name=>'E')}.must_raise Sequel::NoExistingObject
@@ -372,6 +425,7 @@ describe "class_table_inheritance plugin without sti_key with :alias option" do
       end
     end
     class ::Employee < Sequel::Model(@db)
+      private
       def _save_refresh; @values[:id] = 1 end
       def self.columns
         dataset.columns || dataset.opts[:from].first.expression.columns
@@ -502,6 +556,7 @@ describe "class_table_inheritance plugin with duplicate columns" do
       end
     end
     class ::Employee < Sequel::Model(@db)
+      private
       def _save_refresh; @values[:id] = 1 end
       def self.columns
         dataset.columns || dataset.opts[:from].first.expression.columns
@@ -531,6 +586,7 @@ describe "class_table_inheritance plugin with duplicate columns" do
         end
       end
       class ::Employee < Sequel::Model(@db)
+        private
         def _save_refresh; @values[:id] = 1 end
         def self.columns
           dataset.columns || dataset.opts[:from].first.expression.columns
@@ -566,3 +622,167 @@ describe "class_table_inheritance plugin with duplicate columns" do
   end
 end
 
+describe "class_table_inheritance plugin with dataset defined with QualifiedIdentifier" do
+  before do
+    @db = Sequel.mock(:numrows=>1, :autoid=>proc{|sql| 1})
+    def @db.supports_schema_parsing?() true end
+    def @db.schema(table, opts={})
+      {Sequel[:hr][:employees]=>[[:id, {:primary_key=>true, :type=>:integer}], [:name, {:type=>:string}], [:kind, {:type=>:string}]],
+       Sequel[:hr][:managers]=>[[:id, {:type=>:integer}]],
+       Sequel[:hr][:staff]=>[[:id, {:type=>:integer}], [:manager_id, {:type=>:integer}]],
+       Sequel[:hr][:executives]=>[[:id, {:type=>:integer}], [:num_managers, {:type=>:integer}]],
+      }[table.is_a?(Sequel::Dataset) ? table.first_source_table : table]
+    end
+    @db.singleton_class.send(:alias_method, :schema, :schema)
+    @db.extend_datasets do
+      def columns
+        {[Sequel[:hr][:employees]]=>[:id, :name, :kind],
+         [Sequel[:hr][:managers]]=>[:id],
+         [Sequel[:hr][:staff]]=>[:id, :manager_id],
+         [Sequel[:hr][:employees], Sequel[:hr][:managers]]=>[:id, :name, :kind],
+         [Sequel[:hr][:employees], Sequel[:hr][:staff]]=>[:id, :name, :kind, :manager_id],
+         [Sequel[:hr][:employees], Sequel[:hr][:managers], Sequel[:hr][:executives]]=>[:id, :name, :kind, :manager_id, :num_managers],
+        }[opts[:from] + (opts[:join] || []).map{|x| x.table}]
+      end
+    end
+  end
+  after do
+    [:Manager, :Staff, :Employee, :Executive].each{|s| Object.send(:remove_const, s) if Object.const_defined?(s)}
+  end
+
+  describe "with table_map used to qualify subclasses" do
+    before do
+      ::Employee = Class.new(Sequel::Model)
+      ::Employee.db = @db
+      ::Employee.set_dataset(Sequel[:hr][:employees])
+      class ::Employee
+        private
+        def _save_refresh; @values[:id] = 1 end
+        def self.columns
+          dataset.columns || dataset.opts[:from].first.expression.columns
+        end
+        plugin :class_table_inheritance, :table_map=>{:Manager=>Sequel[:hr][:managers],:Staff=>Sequel[:hr][:staff]}
+      end
+      class ::Manager < Employee
+        one_to_many :staff_members, :class=>:Staff
+      end
+      class ::Staff < Employee
+        many_to_one :manager
+      end
+    end
+
+    it "should handle many_to_one relationships correctly" do
+      Manager.dataset = Manager.dataset.with_fetch(:id=>3, :name=>'E')
+      Staff.load(:manager_id=>3).manager.must_equal Manager.load(:id=>3, :name=>'E')
+      @db.sqls.must_equal ['SELECT * FROM (SELECT hr.employees.id, hr.employees.name, hr.employees.kind FROM hr.employees INNER JOIN hr.managers ON (hr.managers.id = hr.employees.id)) AS employees WHERE (id = 3) LIMIT 1']
+    end
+
+    it "should handle one_to_many relationships correctly" do
+      Staff.dataset = Staff.dataset.with_fetch(:id=>1, :name=>'S', :manager_id=>3)
+      Manager.load(:id=>3).staff_members.must_equal [Staff.load(:id=>1, :name=>'S', :manager_id=>3)]
+      @db.sqls.must_equal ['SELECT * FROM (SELECT hr.employees.id, hr.employees.name, hr.employees.kind, hr.staff.manager_id FROM hr.employees INNER JOIN hr.staff ON (hr.staff.id = hr.employees.id)) AS employees WHERE (employees.manager_id = 3)']
+    end
+  end
+
+  describe "without table_map or qualify_tables set" do
+    it "should use a non-qualified subquery in subclasses" do
+      def @db.schema(table, opts={})
+        {Sequel[:hr][:employees]=>[[:id, {:primary_key=>true, :type=>:integer}], [:name, {:type=>:string}], [:kind, {:type=>:string}]],
+         :managers=>[[:id, {:type=>:integer}]],
+        }[table.is_a?(Sequel::Dataset) ? table.first_source_table : table]
+      end
+      @db.extend_datasets do
+        def columns
+          {[Sequel[:hr][:employees]]=>[:id, :name, :kind],
+           [:managers]=>[:id],
+           [Sequel[:hr][:employees], :managers]=>[:id, :name, :kind]
+          }[opts[:from] + (opts[:join] || []).map{|x| x.table}]
+        end
+      end
+      ::Employee = Class.new(Sequel::Model)
+      ::Employee.db = @db
+      ::Employee.set_dataset(Sequel[:hr][:employees])
+      class ::Employee
+        private
+        def _save_refresh; @values[:id] = 1 end
+        def self.columns
+          dataset.columns || dataset.opts[:from].first.expression.columns
+        end
+        plugin :class_table_inheritance
+      end
+      class ::Manager < ::Employee
+      end
+
+      Employee.dataset.sql.must_equal 'SELECT * FROM hr.employees'
+      Manager.dataset.sql.must_equal 'SELECT * FROM (SELECT hr.employees.id, hr.employees.name, hr.employees.kind FROM hr.employees INNER JOIN managers ON (managers.id = hr.employees.id)) AS employees'
+    end
+  end
+
+  describe "with qualify_tables option set" do
+    it "should use a subquery with the same qualifier in subclasses" do
+      ::Employee = Class.new(Sequel::Model)
+      ::Employee.db = @db
+      ::Employee.set_dataset(Sequel[:hr][:employees])
+      class ::Employee
+        private
+        def _save_refresh; @values[:id] = 1 end
+        def self.columns
+          dataset.columns || dataset.opts[:from].first.expression.columns
+        end
+        plugin :class_table_inheritance, :table_map=>{:Staff=>Sequel[:hr][:staff]}, qualify_tables: true
+      end
+      class ::Manager < ::Employee
+        one_to_many :staff_members, :class=>:Staff
+      end
+      class ::Staff < ::Employee
+        many_to_one :manager
+      end
+      class ::Executive < ::Manager
+      end
+
+      Employee.dataset.sql.must_equal 'SELECT * FROM hr.employees'
+      Manager.dataset.sql.must_equal 'SELECT * FROM (SELECT hr.employees.id, hr.employees.name, hr.employees.kind FROM hr.employees INNER JOIN hr.managers ON (hr.managers.id = hr.employees.id)) AS employees'
+      Staff.dataset.sql.must_equal 'SELECT * FROM (SELECT hr.employees.id, hr.employees.name, hr.employees.kind, hr.staff.manager_id FROM hr.employees INNER JOIN hr.staff ON (hr.staff.id = hr.employees.id)) AS employees'
+      Executive.dataset.sql.must_equal 'SELECT * FROM (SELECT hr.employees.id, hr.employees.name, hr.employees.kind, hr.executives.num_managers FROM hr.employees INNER JOIN hr.managers ON (hr.managers.id = hr.employees.id) INNER JOIN hr.executives ON (hr.executives.id = hr.managers.id)) AS employees'
+    end
+  end
+end
+
+describe "class_table_inheritance plugin with schema_caching extension" do
+  before do
+    @db = Sequel.mock(:autoid=>proc{|sql| 1})
+    def @db.supports_schema_parsing?() true end
+    def @db.schema(table, opts={})
+      {:employees=>[[:id, {:primary_key=>true, :type=>:integer}], [:name, {:type=>:string}], [:kind, {:type=>:string}]],
+       :managers=>[[:id, {:type=>:integer}], [:num_staff, {:type=>:integer}] ],
+       :executives=>[[:id, {:type=>:integer}], [:num_managers, {:type=>:integer}]],
+       }[table.is_a?(Sequel::Dataset) ? table.first_source_table : table]
+    end
+  end
+  after do
+    [:Executive, :Manager, :Employee, :Staff].each{|s| Object.send(:remove_const, s) if Object.const_defined?(s)}
+  end
+
+  it "should not query for columns if the schema cache is present and a table_map is given" do
+    class ::Employee < Sequel::Model(@db)
+      plugin :class_table_inheritance, :table_map=>{:Staff=>:employees, :Manager=>:managers, :Executive=>:executives}
+    end
+    class ::Staff < Employee; end
+    class ::Manager < Employee; end
+    class ::Executive < Manager; end
+    Employee.columns.must_equal [:id, :name, :kind]
+    Staff.columns.must_equal [:id, :name, :kind]
+    Manager.columns.must_equal [:id, :name, :kind, :num_staff]
+    Executive.columns.must_equal [:id, :name, :kind, :num_staff, :num_managers]
+    @db.sqls.must_equal []
+  end
+
+  it "should not query for columns if the schema cache is present and no table_map is given" do
+    class ::Employee < Sequel::Model(@db)
+      plugin :class_table_inheritance
+    end
+    class ::Manager < Employee; end
+    class ::Executive < Manager; end
+    @db.sqls.must_equal []
+  end
+end
