@@ -256,7 +256,7 @@ describe "column_encryption plugin" do
 
     need_reencrypt = lambda do
       sql = @model.needing_reencryption.sql
-      [@obj, obj].reject{|o| sql.include?("< '#{o[:enc][0, 4]}'") && sql.include?("> '#{o[:enc][0, 4]}B'") }.length
+      [@obj, obj].count{|o| !(sql.include?("< '#{o[:enc][0, 4]}'") && sql.include?("> '#{o[:enc][0, 4]}B'")) }
     end
 
     need_reencrypt.call.must_equal 0
@@ -395,16 +395,16 @@ describe "column_encryption plugin" do
     @obj[:enc] = enc.dup.tap{|x| x[0] = '%'}
     proc{@obj.enc}.must_raise Sequel::Error # invalid base-64
 
-    @obj[:enc] = enc.dup.tap{|x| x[0,4] = Base64.urlsafe_encode64("\4\0\0")}
+    @obj[:enc] = enc.dup.tap{|x| x[0,4] = "BAAA"} # "\4\0\0" base64
     proc{@obj.enc}.must_raise Sequel::Error # invalid flags
 
-    @obj[:enc] = enc.dup.tap{|x| x[0,4] = Base64.urlsafe_encode64("\0\1\0")}
+    @obj[:enc] = enc.dup.tap{|x| x[0,4] = "AAEA"} # "\0\1\0" base64
     proc{@obj.enc}.must_raise Sequel::Error # invalid reserved byte
 
-    @obj[:enc] = enc.dup.tap{|x| x[0,4] = Base64.urlsafe_encode64("\0\0\1")}
+    @obj[:enc] = enc.dup.tap{|x| x[0,4] = "AAAB"} # "\0\0\1" base64
     proc{@obj.enc}.must_raise Sequel::Error # invalid key id
 
-    @obj[:enc] = enc.dup.tap{|x| x[0,4] = Base64.urlsafe_encode64("\1\0\0")}
+    @obj[:enc] = enc.dup.tap{|x| x[0,4] = "AQAA"} # "\1\0\0" base64
     proc{@obj.enc}.must_raise Sequel::Error # invalid minimum size for searchable
 
     @obj[:enc] = enc.dup.tap{|x| x.slice!(60, 1000)}
