@@ -1,5 +1,7 @@
 require_relative "spec_helper"
 
+require 'delegate'
+
 describe "Sequel::Plugins::DefaultsSetter" do
   before do
     @db = db = Sequel.mock
@@ -7,11 +9,11 @@ describe "Sequel::Plugins::DefaultsSetter" do
     def db.schema(*) [] end
     db.singleton_class.send(:alias_method, :schema, :schema)
     @c = c = Class.new(Sequel::Model(db[:foo]))
-    @c.instance_variable_set(:@db_schema, {:a=>{}})
+    @c.instance_variable_set(:@db_schema, {:a=>{}, :b=>{}})
     @c.plugin :defaults_setter
-    @c.columns :a
+    @c.columns :a, :b
     @pr = proc do |x|
-      db.define_singleton_method(:schema){|*| [[:id, {:primary_key=>true}], [:a, {:ruby_default => x, :primary_key=>false}]]}
+      db.define_singleton_method(:schema){|*| [[:id, {:primary_key=>true}], [:a, {:ruby_default => x, :primary_key=>false}], [:b, {:primary_key=>false}]]}
       db.singleton_class.send(:alias_method, :schema, :schema)
       c.dataset = c.dataset; c
     end
@@ -46,6 +48,46 @@ describe "Sequel::Plugins::DefaultsSetter" do
     (t - Time.now).must_be :<,  1
   end
 
+  it "should not reuse the same default hash for multiple objects" do
+    h = {}
+    c = @pr.call(h)
+    c.new.a.must_equal(h)
+    c.new.a.wont_be_same_as c.new.a
+    c.default_values[:a].must_be_kind_of Proc
+  end
+
+  it "should not reuse the same default array for multiple objects" do
+    h = []
+    c = @pr.call(h)
+    c.new.a.must_equal(h)
+    c.new.a.wont_be_same_as c.new.a
+    c.default_values[:a].must_be_kind_of Proc
+  end
+
+  it "should not reuse the same default hash delegate for multiple objects" do
+    h = DelegateClass(Hash).new({})
+    c = @pr.call(h)
+    c.new.a.must_equal(h)
+    c.new.a.wont_be_same_as c.new.a
+    c.default_values[:a].must_be_kind_of Proc
+  end
+
+  it "should not reuse the same default array delegate for multiple objects" do
+    h = DelegateClass(Array).new([])
+    c = @pr.call(h)
+    c.new.a.must_equal(h)
+    c.new.a.wont_be_same_as c.new.a
+    c.default_values[:a].must_be_kind_of Proc
+  end
+
+  it "should use other object delegates as-is" do
+    h = DelegateClass(Integer).new(1)
+    c = @pr.call(h)
+    c.new.a.must_equal(h)
+    c.new.a.object_id.must_equal c.new.a.object_id
+    c.default_values[:a].object_id.must_equal h.object_id
+  end
+
   it "should handle :callable_default values in schema in preference to :ruby_default" do
     @db.define_singleton_method(:schema) do |*|
       [[:id, {:primary_key=>true}],
@@ -139,6 +181,20 @@ describe "Sequel::Plugins::DefaultsSetter" do
     c.new.a.must_equal 2
   end
 
+  it "should support non-proc/non-method callables" do
+    @pr.call(2)
+    o = Object.new
+    def o.call; 3; end
+    @c.default_values[:a] = o
+    @c.new.a.must_equal 3
+  end
+
+  it "should allow proc default values that depend on other values" do
+    @pr.call(2)
+    @c.default_values[:a] = proc{ |instance| instance.b * 2 }
+    @c.new(b: 5).a.must_equal 10
+  end
+
   it "should set default value upon initialization when plugin loaded without dataset" do
     db = @db
     @c = c = Class.new(Sequel::Model)
