@@ -180,7 +180,11 @@ describe Sequel::Model, ".dataset_module" do
   before do
     @c = Class.new(Sequel::Model(:items))
   end
-  
+ 
+  it "should expose the model class on the dataset module" do
+    @c.dataset_module.model.must_equal @c
+  end
+
   it "should extend the dataset with the module if the model has a dataset" do
     @c.dataset_module{def return_3() 3 end}
     @c.dataset.return_3.must_equal 3
@@ -196,6 +200,12 @@ describe Sequel::Model, ".dataset_module" do
     @c.return_3.must_equal 3
   end
 
+  it "should have methods defined on the class be overridable and allow calling super" do
+    @c.dataset_module{def return_3() 3 end}
+    @c.define_singleton_method(:return_3){super()}
+    @c.return_3.must_equal 3
+  end
+
   it "should add methods defined in the module outside the block to the class" do
     @c.dataset_module.module_eval{def return_3() 3 end}
     @c.return_3.must_equal 3
@@ -538,6 +548,32 @@ describe "Model.qualified_primary_key_hash" do
   end
 end
 
+describe "Model.qualified_primary_key" do
+  before do
+    @c = Class.new(Sequel::Model(:items))
+  end
+  
+  it "should handle a single primary key" do
+    @c.qualified_primary_key.must_equal(Sequel.qualify(:items, :id))
+  end
+
+  it "should handle a composite primary key" do
+    @c.set_primary_key([:id1, :id2])
+    @c.qualified_primary_key.must_equal([Sequel.qualify(:items, :id1), Sequel.qualify(:items, :id2)])
+  end
+
+  it "should raise an error for no primary key" do
+    @c.no_primary_key
+    proc{@c.qualified_primary_key}.must_raise(Sequel::Error)
+  end
+
+  it "should allow specifying a different qualifier" do
+    @c.qualified_primary_key(:apple).must_equal(Sequel.qualify(:apple, :id))
+    @c.set_primary_key([:id1, :id2])
+    @c.qualified_primary_key(:bear).must_equal([Sequel.qualify(:bear, :id1), Sequel.qualify(:bear, :id2)])
+  end
+end
+
 describe "Model.db" do
   before do
     @db = Sequel.mock
@@ -660,13 +696,26 @@ describe Sequel::Model, ".strict_param_setting" do
   end
 
   it "should raise an error if a missing/restricted column/method is accessed" do
-    proc{@c.new(:a=>1)}.must_raise(Sequel::MassAssignmentRestriction)
+    err = proc{@c.new(:a=>1)}.must_raise(Sequel::MassAssignmentRestriction)
+    err.message.must_include("method a= doesn't exist for class ")
+    err.column.must_equal("a")
     proc{@c.create(:a=>1)}.must_raise(Sequel::MassAssignmentRestriction)
     c = @c.new
     proc{c.set(:a=>1)}.must_raise(Sequel::MassAssignmentRestriction)
     proc{c.update(:a=>1)}.must_raise(Sequel::MassAssignmentRestriction)
   end
 
+  it "should add class name to error message" do
+    item = Class.new(Sequel::Model(:items))
+    item.columns :id
+    def item.inspect; 'Item' end
+
+    err = proc{item.new(:a=>1)}.must_raise(Sequel::MassAssignmentRestriction)
+    err.message.must_equal("method a= doesn't exist for class Item")
+    err.model.class.must_equal(item)
+    err.column.must_equal("a")
+  end
+
   it "should be disabled by strict_param_setting = false" do
     @c.strict_param_setting = false
     @c.strict_param_setting.must_equal false
