@@ -5,6 +5,23 @@ describe "Sequel::Model()" do
     @db = Sequel::Model.db
   end
 
+  it "should return a model with a temporary name" do
+    m = Sequel::Model(:blah)
+    m.name.must_equal "Sequel::_Model(:blah)"
+    m.send(:dataset_methods_module).name.must_equal "Sequel::_Model(:blah)::@dataset_methods_module"
+    m.send(:overridable_methods_module).name.must_equal "Sequel::_Model(:blah)::@overridable_methods_module"
+  end if RUBY_VERSION >= '3.3'
+
+  it "should provide correct temporary names when Model.name calls Sequel.synchronize" do
+    m = Sequel::Model(:blah)
+    m.name.must_equal "Sequel::_Model(:blah)"
+    def m.name
+      Sequel.synchronize{super}
+    end
+    m.send(:dataset_methods_module).name.must_equal "Sequel::_Model(:blah)::@dataset_methods_module"
+    m.send(:overridable_methods_module).name.must_equal "Sequel::_Model(:blah)::@overridable_methods_module"
+  end if RUBY_VERSION >= '3.3'
+
   it "should return a model subclass with the given dataset if given a dataset" do
     ds = @db[:blah]
     c = Sequel::Model(ds)
@@ -787,7 +804,7 @@ end
 
 describe "Model#inspect" do
   it "should include the class name and the values" do
-    Sequel::Model.load(:x => 333).inspect.must_equal '#<Sequel::Model @values={:x=>333}>'
+    ['#<Sequel::Model @values={:x=>333}>', '#<Sequel::Model @values={x: 333}>'].must_include Sequel::Model.load(:x => 333).inspect
   end
 end
 
